"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compilePlugin = void 0;
const xtrem_cli_lib_1 = require("@sage/xtrem-cli-lib");
const xtrem_cli_transformers_1 = require("@sage/xtrem-cli-transformers");
const fs = require("fs");
const path = require("path");
const webpack_merge_1 = require("webpack-merge");
const utils_1 = require("../compile/utils");
// use require to fix "This expression is not callable." error during compilation of xtrem-avalara-gateway
const webpack = require("webpack");
const compilePlugin = () => {
    const { executionMode, dir } = xtrem_cli_lib_1.cliContext;
    const buildDir = path.resolve(dir, 'build');
    const localWebpackConfigPath = path.resolve(dir, 'webpack.config.js');
    // Check package file content.
    const packageFileContent = JSON.parse(fs.readFileSync(path.resolve(dir, 'package.json'), 'utf-8'));
    if (packageFileContent.xtremPlugin !== true) {
        (0, xtrem_cli_lib_1.quitWithError)(executionMode, 'Is this package really an Xtrem UI plugin? If so make sure to set the `xtremPlugin` property to true in your package file.');
    }
    // Clean up build directory
    if (fs.existsSync(buildDir)) {
        (0, xtrem_cli_lib_1.printInfo)(executionMode, 'Cleaning build directory...');
        (0, xtrem_cli_lib_1.safeRmSync)(buildDir, { recursive: true });
        (0, xtrem_cli_lib_1.printSuccess)(executionMode, 'Build directory is clear.');
    }
    const localWebpackConfig = fs.existsSync(localWebpackConfigPath)
        ? // eslint-disable-next-line import/no-dynamic-require, global-require
            require(localWebpackConfigPath)
        : {};
    (0, xtrem_cli_lib_1.printInfo)(executionMode, 'Building plugin...');
    // Define the path to the various loaders
    const loaders = {
        tsLoader: require.resolve('ts-loader'),
        urlLoader: require.resolve('url-loader'),
        styleLoader: require.resolve('style-loader'),
        cssLoader: require.resolve('css-loader'),
        sassLoader: require.resolve('sass-loader'),
    };
    // Merge base webpack config with custom configuration declared in the packages root directory
    const config = (0, webpack_merge_1.merge)({
        entry: fs.existsSync(path.resolve(dir, 'lib', 'index.tsx')) ? './lib/index.tsx' : './lib/index.ts',
        mode: 'production',
        devtool: false,
        target: 'web',
        context: dir,
        output: {
            path: buildDir,
            libraryTarget: 'commonjs',
            filename: 'index.js',
        },
        externals: [utils_1.filterExternals],
        plugins: [
            new webpack.optimize.LimitChunkCountPlugin({
                maxChunks: 1,
            }),
        ],
        module: {
            rules: [
                {
                    test: /\.tsx?$/,
                    exclude: /node_modules/,
                    loader: loaders.tsLoader,
                    options: {
                        colors: false,
                        configFile: path.resolve(dir, 'tsconfig.json'),
                        errorFormatter: (errorInfo) => {
                            return JSON.stringify(errorInfo);
                        },
                        getCustomTransformers: () => ({
                            before: [
                                // Injecting message transformer to transform `ui.localize` calls.
                                xtrem_cli_transformers_1.messageTransformer,
                                xtrem_cli_transformers_1.pageMetadataTransformer,
                            ],
                        }),
                    },
                },
                {
                    test: /\.(png|woff|woff2|eot|ttf|svg|gif)$/,
                    use: [
                        {
                            loader: loaders.urlLoader,
                            options: {
                                limit: 100000,
                            },
                        },
                    ],
                },
                {
                    test: /\.css$/,
                    use: [
                        loaders.styleLoader,
                        {
                            loader: loaders.cssLoader,
                            options: {
                                url: {
                                    // Exclude URLs starting with '/absolute/path'
                                    filter: (url) => !url.startsWith('/'),
                                },
                            },
                        },
                    ],
                },
                {
                    test: /\.scss$/,
                    use: [loaders.styleLoader, loaders.cssLoader, loaders.sassLoader],
                    include: path.resolve(dir, 'lib'),
                },
            ],
        },
        resolve: {
            extensions: ['.ts', '.tsx', '.js', '.json'],
        },
    }, localWebpackConfig);
    // Run webpack compilation
    webpack(config).run((err, stats) => {
        const statsObj = stats && stats.toJson();
        if (stats.hasWarnings() && statsObj.warnings)
            statsObj.warnings.forEach(w => {
                (0, xtrem_cli_lib_1.printWarning)(executionMode, w.message);
            });
        if (stats.hasErrors() && statsObj.errors)
            statsObj.errors.forEach(e => {
                (0, xtrem_cli_lib_1.printWarning)(executionMode, e.message);
            });
        if (err) {
            (0, xtrem_cli_lib_1.quitWithError)(executionMode, err.message);
        }
        if (stats.hasErrors()) {
            (0, xtrem_cli_lib_1.quitWithError)(executionMode, 'See compilation errors above.');
        }
        (0, xtrem_cli_transformers_1.mergeTranslationFiles)(dir);
        (0, xtrem_cli_lib_1.printSuccess)(executionMode, 'Plugin is ready.');
        if (executionMode === xtrem_cli_lib_1.ExecutionMode.STANDALONE) {
            process.exit(0);
        }
    });
};
exports.compilePlugin = compilePlugin;
//# sourceMappingURL=index.js.map