"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileFiles = exports.compilePagesAndStickers = exports.getWebpackEntry = exports.isCompilable = exports.filterExternals = exports.checkTypesOnClientArtifactsAndBuildDeclarations = exports.compileClientAndSharedFunctions = void 0;
const xtrem_cli_lib_1 = require("@sage/xtrem-cli-lib");
const xtrem_cli_transformers_1 = require("@sage/xtrem-cli-transformers");
const path = require("path");
const rxjs_1 = require("rxjs");
const ts = require("typescript");
const webpack = require("webpack");
const server_compilation_1 = require("./server-compilation");
const xtrem_client_plugin_1 = require("./xtrem-client-plugin");
const TerserPlugin = require('terser-webpack-plugin');
const compileClientAndSharedFunctions = (executionMode, dir) => {
    // transpile client and shared functions
    (0, xtrem_cli_lib_1.printInfo)(executionMode, 'Compiling client and shared functions...');
    const clientAndSharedProgram = (0, server_compilation_1.getTsProgram)(dir, {
        configFileName: 'tsconfig-artifacts.json',
        includeOverride: ['lib/client-functions', 'lib/shared-functions'],
        tsconfigOverride: {
            declaration: false,
            composite: false,
            declarationMap: false,
        },
    });
    const clientAndSharedDiagnosticsEmitResult = clientAndSharedProgram.emit(undefined, undefined, undefined, undefined, {
        before: [xtrem_cli_transformers_1.decimalTransformer, xtrem_cli_transformers_1.messageTransformer, xtrem_cli_transformers_1.pageMetadataTransformer],
    });
    const clientAndSharedDiagnostics = ts
        .getPreEmitDiagnostics(clientAndSharedProgram)
        .concat(clientAndSharedDiagnosticsEmitResult.diagnostics);
    clientAndSharedDiagnostics.forEach((0, server_compilation_1.printCompilerError)(executionMode));
    if (clientAndSharedDiagnostics.length === 0) {
        (0, xtrem_cli_lib_1.printSuccess)(executionMode, 'Shared and client functions compiled.');
    }
    else {
        (0, xtrem_cli_lib_1.quitWithError)(executionMode, 'Failed to compile shared and client functions.');
    }
};
exports.compileClientAndSharedFunctions = compileClientAndSharedFunctions;
const checkTypesOnClientArtifactsAndBuildDeclarations = (executionMode, dir) => {
    // transpile client and shared functions
    (0, xtrem_cli_lib_1.printInfo)(executionMode, 'Type checking client...');
    const diagnosticsProgram = (0, server_compilation_1.getTsProgram)(dir, {
        configFileName: 'tsconfig-artifacts.json',
        excludeOverride: ['build/*/*.d.ts'],
        tsconfigOverride: {
            strictNullChecks: false,
            skipLibCheck: true,
            emitDeclarationOnly: true,
        },
    });
    const diagnosticsResult = diagnosticsProgram.emit();
    const diagnostics = ts.getPreEmitDiagnostics(diagnosticsProgram).concat(diagnosticsResult.diagnostics);
    diagnostics.forEach((0, server_compilation_1.printCompilerError)(executionMode));
    if (diagnostics.length === 0) {
        (0, xtrem_cli_lib_1.printSuccess)(executionMode, 'Type checks passed.');
    }
    else {
        (0, xtrem_cli_lib_1.quitWithError)(executionMode, 'Failed to compile client.');
    }
};
exports.checkTypesOnClientArtifactsAndBuildDeclarations = checkTypesOnClientArtifactsAndBuildDeclarations;
const filterExternals = ({ request }) => {
    /**
     * BL: The `@sage/xtrem-ui` is part of the front-end application which loads the pages which we compile here.
     * Therefore this dependency can be injected instead of packaging it up, so here we just return an empty module
     * instead of actually packaging it up.
     */
    if (/@sage\/xtrem-ui/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (/@sage\/xtrem-async-helper/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (/@sage\/xtrem-i18n/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (/@sage\/xtrem-date-time/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (/@sage\/xtrem-decimal/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (/@sage\/xtrem-shared/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (/@sage\/xtrem-client/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (/@sage\/xtrem-core/.test(request)) {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (request === 'lodash') {
        return Promise.resolve(`commonjs ${request}`);
    }
    if (request === 'react') {
        return Promise.resolve(`commonjs ${request}`);
    }
    return Promise.resolve();
};
exports.filterExternals = filterExternals;
const isCompilable = (filePath) => {
    const basename = path.basename(filePath);
    return basename !== 'index.ts' && basename !== '_index.ts' && basename.endsWith('.ts');
};
exports.isCompilable = isCompilable;
const getWebpackEntry = (executionMode, dir, isOnlyChanged) => {
    return xtrem_client_plugin_1.XtremClientPlugin.getEntries(executionMode, dir, [
        './lib/pages/*.ts',
        './lib/stickers/*.ts',
        './lib/page-extensions/*.ts',
        './lib/page-fragments/*.ts',
        './lib/widgets/*.ts',
    ], {
        ignore: '**/index.ts',
        cwd: dir,
        absolute: true,
        realpath: true,
    }, isOnlyChanged);
};
exports.getWebpackEntry = getWebpackEntry;
const compilePagesAndStickers = ({ executionMode, buildDir, isProd, isInstrumented, isWatch = false, isOnlyChanged = false, packageName, }) => {
    const entry = (0, exports.getWebpackEntry)(executionMode, buildDir, isOnlyChanged);
    const targetDir = path.resolve(buildDir, 'build/lib').toLowerCase();
    const output = {
        path: targetDir,
        filename: '[name].js',
        // https://webpack.js.org/configuration/output/#librarytarget-assign-properties
        library: {
            name: 'xtremArtifact',
            type: 'assign-properties',
        },
        devtoolNamespace: packageName,
        devtoolModuleFilenameTemplate: 'webpack://[namespace]/[resource-path]',
    };
    const packagingMode = isProd ? xtrem_cli_lib_1.PackagingMode.PROD : xtrem_cli_lib_1.PackagingMode.DEV;
    return (0, exports.compileFiles)({
        buildDir,
        entry,
        output,
        packagingMode,
        isInstrumented,
        options: isWatch ? { watch: true } : {},
    });
};
exports.compilePagesAndStickers = compilePagesAndStickers;
const compileFiles = ({ buildDir, entry, output, packagingMode = xtrem_cli_lib_1.PackagingMode.DEV, isInstrumented, options = {}, }) => {
    const devMode = packagingMode === xtrem_cli_lib_1.PackagingMode.DEV;
    const isWatchEnabled = (options && options.watch) || false;
    let devtool = 'source-map';
    if (!isInstrumented) {
        devtool = devMode ? 'eval-source-map' : false;
    }
    // Define the path to the various loaders
    const loaders = {
        tsLoader: require.resolve('ts-loader'),
        threadLoader: require.resolve('thread-loader'),
    };
    const config = {
        entry: entry,
        mode: devMode ? 'development' : 'production',
        devtool,
        performance: {
            hints: false,
            maxEntrypointSize: 512000,
            maxAssetSize: 512000,
        },
        optimization: packagingMode === xtrem_cli_lib_1.PackagingMode.PROD
            ? {
                minimize: true,
                minimizer: [
                    new TerserPlugin({
                        terserOptions: {
                            keep_classnames: true,
                            keep_fnames: true,
                        },
                    }),
                ],
            }
            : undefined,
        ...(isWatchEnabled
            ? {
                watchOptions: {
                    aggregateTimeout: 600,
                },
                devServer: {
                    contentBase: path.resolve(buildDir, 'build/lib').toLowerCase(),
                    compress: true,
                    publicPath: '/',
                    port: 4000,
                    host: '0.0.0.0',
                    historyApiFallback: {
                        index: '/',
                    },
                    proxy: [
                        {
                            target: 'http://localhost:8240',
                        },
                    ],
                },
            }
            : {}),
        output,
        plugins: [
            new xtrem_client_plugin_1.XtremClientPlugin(buildDir),
            new webpack.ProvidePlugin({
                Buffer: ['buffer', 'Buffer'],
            }),
        ],
        externals: [exports.filterExternals],
        module: {
            rules: [
                ...(!isWatchEnabled
                    ? [
                        {
                            loader: loaders.threadLoader,
                            options: {
                                workerParallelJobs: 50,
                                workerNodeArgs: ['--max-old-space-size=1024'],
                                poolRespawn: false,
                                poolTimeout: 2000,
                                poolParallelJobs: 50,
                            },
                        },
                    ]
                    : []),
                {
                    test: /\.ts$/,
                    exclude: /node_modules/,
                    loader: loaders.tsLoader,
                    options: {
                        happyPackMode: !isWatchEnabled,
                        colors: false,
                        configFile: path.resolve(buildDir, 'tsconfig-artifacts.json'),
                        errorFormatter: (errorInfo) => {
                            return JSON.stringify(errorInfo);
                        },
                        getCustomTransformers: require.resolve('./transformers'),
                    },
                },
            ],
        },
        resolve: {
            fallback: {
                // used in xtrem-reporting
                buffer: require.resolve('buffer/'),
            },
            extensions: ['.ts', '.tsx', '.js', '.json'],
        },
        ...options,
    };
    // TODO: remove any but compile fails on xtrem-avalara-gateway
    return runWebpack({ compiler: webpack(config), isWatchEnabled });
};
exports.compileFiles = compileFiles;
const runWebpack = ({ compiler, isWatchEnabled: watchMode = false }) => {
    if (watchMode) {
        return new rxjs_1.Observable((obs) => {
            compiler.watch({ ignored: /node_modules/, poll: true }, (error, stats) => {
                obs.next({ stats, error });
            });
        });
    }
    return new rxjs_1.Observable((obs) => {
        compiler.run((error, stats) => {
            obs.next({ stats, error });
        });
    });
};
//# sourceMappingURL=client-compilation.js.map