"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.webpackObserver = exports.printCompilerErrorMessage = exports.watchClientArtifacts = void 0;
const xtrem_cli_lib_1 = require("@sage/xtrem-cli-lib");
const xtrem_core_1 = require("@sage/xtrem-core");
const glob_1 = require("glob");
const utils_1 = require("./utils");
const watchClientArtifacts = (executionMode, buildDir) => {
    const globOptions = { cwd: buildDir, absolute: true, realpath: true };
    const pages = glob_1.glob.sync('./lib/pages/*.ts', globOptions).filter(utils_1.isCompilable);
    const stickers = glob_1.glob.sync('./lib/stickers/*.ts', globOptions).filter(utils_1.isCompilable);
    if (pages.length <= 0 && stickers.length <= 0) {
        (0, xtrem_cli_lib_1.printInfo)(executionMode, 'No client side artifacts found.');
        return undefined;
    }
    return (0, utils_1.compilePagesAndStickers)({
        executionMode,
        buildDir,
        isProd: false,
        isInstrumented: false,
        isWatch: true,
        isOnlyChanged: false,
        packageName: buildDir.split('/').pop() ?? '',
    });
};
exports.watchClientArtifacts = watchClientArtifacts;
const printCompilerErrorMessage = (executionMode, error) => {
    console.log(error);
    try {
        const lines = error.stack?.split('\n');
        const errorObject = JSON.parse(lines[lines.length - 1]);
        (0, xtrem_cli_lib_1.printWarning)(executionMode, `${errorObject.file}:${errorObject.line}:${errorObject.character}\n${errorObject.content}`);
    }
    catch (e) {
        (0, xtrem_cli_lib_1.printWarning)(executionMode, error.message);
    }
};
exports.printCompilerErrorMessage = printCompilerErrorMessage;
const webpackObserver = ({ executionMode, deployedApp, dir, }) => ({
    next: result => {
        const statsObj = result.stats && result.stats.toJson();
        const hasWarnings = statsObj && statsObj.warnings && statsObj.warnings.length > 0;
        const hasErrors = statsObj && statsObj.errors && statsObj.errors.length > 0;
        const isCompilationSuccessful = !result.error && !hasWarnings && !hasErrors;
        if (isCompilationSuccessful) {
            if (deployedApp.app) {
                xtrem_core_1.ArtifactManager.clearStored();
                (0, xtrem_cli_lib_1.printSuccess)(executionMode, 'Webpack compiler finished successfully. Please refresh your webpage to see the changes.');
                (0, xtrem_cli_lib_1.broadcastCompilerSuccessMessage)('client');
                return;
            }
            (0, xtrem_cli_lib_1.printSuccess)(executionMode, 'Webpack compiler finished successfully.');
            (0, xtrem_cli_lib_1.broadcastCompilerSuccessMessage)('client');
        }
        else {
            if (!statsObj) {
                return;
            }
            if (hasWarnings)
                statsObj.warnings?.forEach(w => {
                    (0, xtrem_cli_lib_1.addTsLoaderErrorToDispatchQueue)(w.message, dir);
                    (0, exports.printCompilerErrorMessage)(executionMode, w);
                });
            if (hasErrors)
                statsObj.errors?.forEach(e => {
                    (0, xtrem_cli_lib_1.addTsLoaderErrorToDispatchQueue)(e.message, dir);
                    (0, exports.printCompilerErrorMessage)(executionMode, e);
                });
            (0, xtrem_cli_lib_1.printWarning)(executionMode, `${(0, xtrem_cli_lib_1.plural)('error was', 'errors were', (statsObj.errors ?? []).length)} found during Webpack compilation.\nHot-reloading is enabled though, you can issue another build just by re-saving any page/sticker file.\nNote that the Xtrem server will be restarted automatically upon every successful build.`);
        }
    },
    error: err => (0, xtrem_cli_lib_1.printWarning)(executionMode, `Unexpected error from watch-client: ${err}`),
    complete: () => { },
});
exports.webpackObserver = webpackObserver;
//# sourceMappingURL=watch-client.js.map