"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.notificationListenerTransformer = void 0;
exports.isNodeLikeArtifactFile = isNodeLikeArtifactFile;
const path = require("path");
const ts = require("typescript");
const __1 = require("..");
const logger = require("./notification-listener-routing/logger");
const routing_builder_1 = require("./notification-listener-routing/routing-builder");
const notificationListenerTransformer = (context) => {
    const rootDir = context.getCompilerOptions().rootDir;
    const routingBuilder = new routing_builder_1.RoutingBuilder(rootDir);
    return sourceFile => {
        if (!isNodeLikeArtifactFile(sourceFile)) {
            return sourceFile;
        }
        routingBuilder.setVisitingFile(sourceFile.fileName);
        const visitor = (node) => {
            let visitingNodeDecorator = false;
            try {
                if (isListenerDecorator(node, 'notificationListener')) {
                    const listener = routingBuilder.createListener(node);
                    routingBuilder.updateRouting(listener);
                    routingBuilder.writeRoutingData();
                }
                else if (isListenerDecorator(node, 'asyncMutation') || isListenerDecorator(node, 'bulkMutation')) {
                    const listener = routingBuilder.createAsyncMutationListener(node);
                    routingBuilder.updateRouting(listener);
                    routingBuilder.writeRoutingData();
                }
                else if (isNodeDecorator(node) || (0, __1.isSubNodeDecorator)(node)) {
                    visitingNodeDecorator = true;
                    routingBuilder.setVisitingNodeDecorator(node);
                    routingBuilder.createListeners();
                }
                return ts.visitEachChild(node, visitor, context);
            }
            catch (err) {
                const { line, character } = sourceFile.getLineAndCharacterOfPosition(node.getStart());
                logger.error(err);
                logger.error(`fileName: ${sourceFile.fileName}:${line + 1}:${character + 1}`);
                logger.error(`nodeText: ${node.getText()}`);
                throw err;
            }
            finally {
                if (visitingNodeDecorator) {
                    routingBuilder.setVisitingNodeDecorator(undefined);
                }
            }
        };
        return ts.visitNode(sourceFile, visitor);
    };
};
exports.notificationListenerTransformer = notificationListenerTransformer;
function isNodeLikeArtifactFile(file) {
    const filePath = file.fileName;
    const fileName = path.basename(filePath, '.ts');
    return (fileName !== 'index' &&
        fileName !== '_index' &&
        !/\/(x3-services|wh-services|wms|test|fixtures)\//.test(filePath) &&
        /[/\\]lib[/\\](nodes|node-extensions)/.test(filePath));
}
function isListenerDecorator(node, name) {
    return (ts.isCallExpression(node) &&
        ts.isDecorator(node.parent) &&
        ts.isPropertyAccessExpression(node.expression) &&
        node.expression.name.escapedText === name);
}
function isNodeDecorator(node) {
    return (ts.isDecorator(node) &&
        ts.isClassDeclaration(node.parent) &&
        ts.isCallExpression(node.expression) &&
        ts.isPropertyAccessExpression(node.expression.expression) &&
        node.expression.expression.getText() === 'decorators.node');
}
//# sourceMappingURL=notification-listener-transformer.js.map